class RomanNumeralGenerator
  VALUES = [ 
    [1000,"M"],
    [900,"CM"],
    [500,"D"],
    [400,"CD"],
    [100,"C"],
    [90,"XC"],
    [50,"L"],
    [40,"XL"],
    [10,"X"],
    [9,"IX"],
    [5,"V"],
    [4,"IV"],
    [1,"I"]
  ]

  def generate(number)
    check_range(number)
    arabic_to_roman(number)
  end

  def is_out_of_range?(number)
    number < 1 || number > 3999
  end

  private

  def arabic_to_roman(number)
    [].tap do |answer|
      VALUES.each do |numeral, roman|
        while number >= numeral
          answer << roman
          number -= numeral
        end
      end
    end.join
  end

  def check_range(number)
    raise RangeError.new("Use only number in [1..3999] range") if is_out_of_range?(number)
  end
end
